// ignore_for_file: public_member_api_docs, always_use_package_imports, non_constant_identifier_names, lines_longer_than_80_chars, avoid_positional_boolean_parameters, unused_element, use_setters_to_change_properties

import 'package:healthrian_common_support/healthrian_common_support.dart';

class DataFilter {
  DataFilter(int channel, {bool is250Hz = true}) {
    _filterMap[DcBlock.on] = List.generate(channel, (_) => DigitalFilter());

    /// iir
    for (final filterType in filterTypes) {
      final coefficients = is250Hz ? IIRData.freq250hz : IIRData.freq500hz;
      final coefficient = coefficients[filterType];
      if (coefficient == null) continue;
      _filterMap[filterType] = List.generate(
          channel, (_) => IIRFilter(filterType: filterType, is250Hz: is250Hz));
    }

    /// fir
    for (final filterType in filterTypes) {
      final coefficients = is250Hz ? FIRData.freq250hz : FIRData.freq500hz;
      final coefficient = coefficients[filterType];
      if (coefficient == null) continue;
      _filterMap[filterType] = List.generate(
          channel, (_) => FIRFilter(filterType: filterType, is250Hz: is250Hz));
    }

    /// adaptive
    _filterMap[Adaptive.on] =
        List.generate(channel, (_) => AdaptiveFilter(is250Hz: is250Hz));
  }

  ///
  DcBlock dcBlock = DcBlock.on;

  ///
  LowPass lowPass = LowPass.lpf40;
  HighPass highPass = HighPass.hpf01;
  BandStop bandStop = BandStop.bsf60;

  ///
  FirBandStop firBandStop = FirBandStop.off;

  ///
  Adaptive adaptive = Adaptive.off;

  final Map<Enum, List<FilterInterface>> _filterMap = {};

  List<Enum> get _list =>
      [dcBlock, lowPass, highPass, bandStop, firBandStop, adaptive];

  Map<Enum, bool> get _flagMap => {for (final e in _list) e: true};

  double applyFilter(
    int channel,
    double resultData,
    bool leadState,
  ) {
    var filterResultData = resultData;
    if (leadState) {
      if (dcBlock != DcBlock.off) {
        filterResultData =
            _filterMap[dcBlock]![channel].apply(filterResultData);
      }

      if (lowPass != LowPass.off) {
        filterResultData =
            _filterMap[lowPass]![channel].apply(filterResultData);
      }

      if (highPass != HighPass.off) {
        filterResultData =
            _filterMap[highPass]![channel].apply(filterResultData);
      }

      if (bandStop != BandStop.off) {
        filterResultData =
            _filterMap[bandStop]![channel].apply(filterResultData);
      }

      if (firBandStop != FirBandStop.off) {
        filterResultData =
            _filterMap[firBandStop]![channel].apply(filterResultData);
      }

      if (adaptive != Adaptive.off) {
        filterResultData =
            _filterMap[adaptive]![channel].apply(filterResultData);
      }
    }

    return filterResultData;
  }
}
