// ignore_for_file: public_member_api_docs, always_use_package_imports, non_constant_identifier_names

import 'package:json_annotation/json_annotation.dart';

import 'iir_filter.dart';

part 'iir_filter_class.g.dart';

// TODO: sampling rate 조절 기능 추가
@JsonSerializable()
class IIRFilterClass {
  factory IIRFilterClass.fromJson(Map<String, dynamic> json) =>
      _$IIRFilterClassFromJson(json);

  Map<String, dynamic> toJson() => _$IIRFilterClassToJson(this);

  IIRFilterClass({
    required this.numer,
    required this.denom,
    required this.sample_cnt,
  });

  IIRFilterClass.initialize(String filterSpec) {
    if (filterSpec == IIRConstant.IIR_HPF_0_05) {
      numer = IIRCoefficient.IIR_HPF_0_05_NUMERATOR;
      denom = IIRCoefficient.IIR_HPF_0_05_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_HPF_0_1) {
      numer = IIRCoefficient.IIR_HPF_0_1_NUMERATOR;
      denom = IIRCoefficient.IIR_HPF_0_1_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_HPF_0_5) {
      numer = IIRCoefficient.IIR_HPF_0_5_NUMERATOR;
      denom = IIRCoefficient.IIR_HPF_0_5_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_LPF_40) {
      numer = IIRCoefficient.IIR_LPF_40_NUMERATOR;
      denom = IIRCoefficient.IIR_LPF_40_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_LPF_100) {
      numer = IIRCoefficient.IIR_LPF_100_NUMERATOR;
      denom = IIRCoefficient.IIR_LPF_100_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_BSF_50) {
      numer = IIRCoefficient.IIR_BSF_50_NUMERATOR;
      denom = IIRCoefficient.IIR_BSF_50_DENOMINATOR;
    } else if (filterSpec == IIRConstant.IIR_BSF_60) {
      numer = IIRCoefficient.IIR_BSF_60_NUMERATOR;
      denom = IIRCoefficient.IIR_BSF_60_DENOMINATOR;
    } else {
      numer = IIRCoefficient.IIR_BSF_60_NUMERATOR;
      denom = IIRCoefficient.IIR_BSF_60_DENOMINATOR;
    }
  }

  late List<double> numer;
  late List<double> denom;
  final List<double> x_val = [0, 0, 0];
  final List<double> y_val = [0, 0, 0];

  int sample_cnt = 0;

  double apply(double inputVal) {
    var y1 = 0.0;
    if (sample_cnt == 3) {
      for (var ptr = 0; ptr < 2; ptr++) {
        x_val[ptr] = x_val[ptr + 1];
      }
      x_val[2] = inputVal;

      for (var ptr = 0; ptr < 3; ptr++) {
        y1 += numer[ptr] * x_val[2 - ptr];
      }

      for (var ptr = 1; ptr < 3; ptr++) {
        y1 -= denom[ptr] * y_val[2 - ptr + 1];
      }

      for (var ptr = 0; ptr < 2; ptr++) {
        y_val[ptr] = y_val[ptr + 1];
      }

      // _sample_cnt = 0;
      y_val[2] = y1;

      return y1;
    } else {
      x_val[sample_cnt] = inputVal;

      for (var ptr = 0; ptr < sample_cnt + 1; ptr++) {
        y1 += numer[ptr] * x_val[sample_cnt - ptr];

        if (ptr > 0) {
          y1 -= denom[ptr] * y_val[sample_cnt - ptr];
        }
      }

      y_val[sample_cnt] = y1;
      sample_cnt += 1;

      return y1;
    }
  }
}
