import 'package:healthrian_common_support/healthrian_common_support.dart';

class FIRFilter extends FilterInterface {
  int length = 0;
  List<double>? delayLine;
  List<double>? impulseResponse;
  int count = 0;

  FIRFilter({required Enum filterType, bool is250Hz = true}) {
    final freq = is250Hz ? FIRData.freq250hz : FIRData.freq500hz;
    final filter = freq[filterType];
    if (filter == null) return;

    length = filter.length;
    impulseResponse = [...filter];
    delayLine = List.filled(length, 0);
  }

  double apply(double input) {
    if (length == 0) return input;
    delayLine![count] = input;
    double result = 0.0;
    int index = count;
    for (int i = 0; i < length; i++) {
      result += impulseResponse![i] * delayLine![index--];
      if (index < 0) index = length - 1;
    }
    if (++count >= length) count = 0;
    return result;
  }
}
