import 'package:healthrian_common_support/healthrian_common_support.dart';

class IIRFilter extends FilterInterface {
  List<double>? numerator;
  List<double>? denominator;

  final List<double> x_val = [0, 0, 0];
  final List<double> y_val = [0, 0, 0];

  IIRFilter({required Enum filterType, bool is250Hz = true}) {
    final freq = is250Hz ? IIRData.freq250hz : IIRData.freq500hz;
    final filter = freq[filterType];
    if (filter == null) return;

    numerator = [...filter.numerator];
    denominator = [...filter.denominator];
  }

  int sample_cnt = 0;

  double apply(double input) {
    if (numerator == null || denominator == null) return input;
    var y1 = 0.0;
    if (sample_cnt == 3) {
      for (var ptr = 0; ptr < 2; ptr++) {
        x_val[ptr] = x_val[ptr + 1];
      }
      x_val[2] = input;

      for (var ptr = 0; ptr < 3; ptr++) {
        y1 += numerator![ptr] * x_val[2 - ptr];
      }

      for (var ptr = 1; ptr < 3; ptr++) {
        y1 -= denominator![ptr] * y_val[2 - ptr + 1];
      }

      for (var ptr = 0; ptr < 2; ptr++) {
        y_val[ptr] = y_val[ptr + 1];
      }

      // _sample_cnt = 0;
      y_val[2] = y1;

      return y1;
    } else {
      x_val[sample_cnt] = input;

      for (var ptr = 0; ptr < sample_cnt + 1; ptr++) {
        y1 += numerator![ptr] * x_val[sample_cnt - ptr];

        if (ptr > 0) {
          y1 -= denominator![ptr] * y_val[sample_cnt - ptr];
        }
      }

      y_val[sample_cnt] = y1;
      sample_cnt += 1;

      return y1;
    }
  }
}
